@@ -1,9 +1,10 @@
+{{=@@ @@=}}
 %%%-------------------------------------------------------------------
-%% @doc {{name}} top level supervisor.
+%% @doc @@name@@ top level supervisor.
 %% @end
 %%%-------------------------------------------------------------------
 
--module({{name}}_sup).
+-module(@@name@@_sup).
 
 -behaviour(supervisor).
 
@@ -26,9 +27,12 @@ start_link() ->
 %% Supervisor callbacks
 %%====================================================================
 
+%% Child :: #{id => Id, start => {M, F, A}}
+%% Optional keys are restart, shutdown, type, modules.
+%% Before OTP 18 tuples must be used to specify a child. e.g.
 %% Child :: {Id,StartFunc,Restart,Shutdown,Type,Modules}
 init([]) ->
-    {ok, { {one_for_all, 0, 1}, []} }.
+    {ok, {{one_for_all, 0, 1}, []}}.
 
 %%====================================================================
 %% Internal functions
