@@ -1,59 +1,6 @@
 -module(rebar_compile_SUITE).
 
--export([suite/0,
-         init_per_suite/1,
-         end_per_suite/1,
-         init_per_testcase/2,
-         end_per_testcase/2,
-         init_per_group/2,
-         end_per_group/2,
-         all/0,
-         groups/0,
-         build_basic_app/1, paths_basic_app/1, clean_basic_app/1,
-         build_release_apps/1, paths_release_apps/1, clean_release_apps/1,
-         build_checkout_apps/1, paths_checkout_apps/1,
-         build_checkout_deps/1, paths_checkout_deps/1,
-         build_basic_srcdirs/1, paths_basic_srcdirs/1,
-         build_release_srcdirs/1, paths_release_srcdirs/1,
-         build_unbalanced_srcdirs/1, paths_unbalanced_srcdirs/1,
-         build_basic_extra_dirs/1, paths_basic_extra_dirs/1, clean_basic_extra_dirs/1,
-         build_release_extra_dirs/1, paths_release_extra_dirs/1, clean_release_extra_dirs/1,
-         build_unbalanced_extra_dirs/1, paths_unbalanced_extra_dirs/1,
-         build_extra_dirs_in_project_root/1,
-         paths_extra_dirs_in_project_root/1,
-         clean_extra_dirs_in_project_root/1,
-         recompile_when_hrl_changes/1,
-         recompile_when_included_hrl_changes/1,
-         recompile_when_opts_included_hrl_changes/1,
-         recompile_when_opts_change/1,
-         dont_recompile_when_opts_dont_change/1,
-         dont_recompile_yrl_or_xrl/1,
-         deps_in_path/1,
-         delete_beam_if_source_deleted/1,
-         checkout_priority/1,
-         highest_version_of_pkg_dep/1,
-         parse_transform_test/1,
-         erl_first_files_test/1,
-         mib_test/1,
-         umbrella_mib_first_test/1,
-         only_default_transitive_deps/1,
-         clean_all/1,
-         override_deps/1,
-         profile_override_deps/1,
-         profile_deps/1,
-         deps_build_in_prod/1,
-         include_file_relative_to_working_directory/1,
-         include_file_in_src/1,
-         include_file_relative_to_working_directory_test/1,
-         include_file_in_src_test/1,
-         include_file_in_src_test_multiapp/1,
-         dont_recompile_when_erl_compiler_options_env_does_not_change/1,
-         recompile_when_erl_compiler_options_env_changes/1,
-         always_recompile_when_erl_compiler_options_set/1,
-         recompile_when_parse_transform_inline_changes/1,
-         recompile_when_parse_transform_as_opt_changes/1,
-         recursive/1,no_recursive/1,
-         regex_filter_skip/1, regex_filter_regression/1]).
+-compile(export_all).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("eunit/include/eunit.hrl").
@@ -75,9 +22,13 @@ all() ->
      delete_beam_if_source_deleted,
      deps_in_path, checkout_priority, highest_version_of_pkg_dep,
      parse_transform_test, erl_first_files_test, mib_test,
-     umbrella_mib_first_test, only_default_transitive_deps,
-     clean_all, override_deps, profile_override_deps, deps_build_in_prod,
-     profile_override_deps, profile_deps, deps_build_in_prod,
+     umbrella_mib_first_test, only_default_transitive_deps, clean_all,
+     profile_deps, deps_build_in_prod, only_deps,
+     override_deps, override_add_deps, override_del_deps,
+     override_opts, override_add_opts, override_del_opts,
+     apply_overrides_exactly_once,
+     profile_override_deps, profile_override_add_deps, profile_override_del_deps,
+     profile_override_opts, profile_override_add_opts, profile_override_del_opts,
      include_file_relative_to_working_directory, include_file_in_src,
      include_file_relative_to_working_directory_test, include_file_in_src_test,
      include_file_in_src_test_multiapp,
@@ -87,7 +38,8 @@ all() ->
      recursive, no_recursive,
      always_recompile_when_erl_compiler_options_set,
      dont_recompile_when_erl_compiler_options_env_does_not_change,
-     recompile_when_erl_compiler_options_env_changes].
+     recompile_when_erl_compiler_options_env_changes,
+     rebar_config_os_var].
 
 groups() ->
     [{basic_app, [], [build_basic_app, paths_basic_app, clean_basic_app]},
@@ -823,7 +775,7 @@ recompile_when_opts_change(Config) ->
 
     rebar_test_utils:create_config(AppDir, [{erl_opts, [{d, some_define}]}]),
 
-    rebar_test_utils:run_and_check(Config, [], ["compile"], {ok, [{app, Name}]}),
+    rebar_test_utils:run_and_check(Config, [{erl_opts, [{d, some_define}]}], ["compile"], {ok, [{app, Name}]}),
 
     {ok, NewFiles} = rebar_utils:list_dir(EbinDir),
     NewModTime = [filelib:last_modified(filename:join([EbinDir, F]))
@@ -880,19 +832,55 @@ dont_recompile_yrl_or_xrl(Config) ->
         "Erlang code.",
     ok = ec_file:write(Xrl, XrlBody),
 
-    XrlBeam = filename:join([AppDir, "ebin", filename:basename(Xrl, ".xrl") ++ ".beam"]),
+    Yrl = filename:join([AppDir, "src", "not_a_real_yrl_" ++ Name ++ ".yrl"]),
+    ok = filelib:ensure_dir(Yrl),
+    YrlBody = ["Nonterminals E T F.\n"
+               "Terminals '+' '*' '(' ')' number.\n"
+               "Rootsymbol E.\n"
+               "E -> E '+' T: {'$2', '$1', '$3'}.\n"
+               "E -> T : '$1'.\n"
+               "T -> T '*' F: {'$2', '$1', '$3'}.\n"
+               "T -> F : '$1'.\n"
+               "F -> '(' E ')' : '$2'.\n"
+               "F -> number : '$1'.\n"],
+    ok = ec_file:write(Yrl, YrlBody),
 
-    rebar_test_utils:run_and_check(Config, [], ["compile"], {ok, [{app, Name}]}),
+    XrlErl = filename:join([AppDir, "src", filename:basename(Xrl, ".xrl") ++ ".erl"]),
+    YrlErl = filename:join([AppDir, "src", filename:basename(Yrl, ".yrl") ++ ".erl"]),
 
-    ModTime = filelib:last_modified(XrlBeam),
+    EbinDir = filename:join([AppDir, "_build", "default", "lib", Name, "ebin"]),
+    XrlBeam = filename:join([EbinDir, filename:basename(Xrl, ".xrl") ++ ".beam"]),
+    YrlBeam = filename:join([EbinDir, filename:basename(Yrl, ".yrl") ++ ".beam"]),
+
+    Hrl = filename:join([AppDir, "include", "some_header.hrl"]),
+    ok = filelib:ensure_dir(Hrl),
+    HrlBody = yeccpre_hrl(),
+    ok = ec_file:write(Hrl, HrlBody),
+    RebarConfig = [{yrl_opts, [{includefile, "include/some_header.hrl"}]}],
+
+    rebar_test_utils:run_and_check(Config, RebarConfig, ["compile"], {ok, [{app, Name}]}),
+
+    XrlModTime = filelib:last_modified(XrlErl),
+    YrlModTime = filelib:last_modified(YrlErl),
+
+    XrlBeamModTime = filelib:last_modified(XrlBeam),
+    YrlBeamModTime = filelib:last_modified(YrlBeam),
 
     timer:sleep(1000),
 
-    rebar_test_utils:run_and_check(Config, [], ["compile"], {ok, [{app, Name}]}),
+    rebar_test_utils:run_and_check(Config, RebarConfig, ["compile"], {ok, [{app, Name}]}),
 
-    NewModTime = filelib:last_modified(XrlBeam),
+    NewXrlModTime = filelib:last_modified(XrlErl),
+    NewYrlModTime = filelib:last_modified(YrlErl),
 
-    ?assert(ModTime == NewModTime).
+    NewXrlBeamModTime = filelib:last_modified(XrlBeam),
+    NewYrlBeamModTime = filelib:last_modified(YrlBeam),
+
+    ?assert(XrlBeamModTime == NewXrlBeamModTime),
+    ?assert(YrlBeamModTime == NewYrlBeamModTime),
+
+    ?assert(XrlModTime == NewXrlModTime),
+    ?assert(YrlModTime == NewYrlModTime).
 
 delete_beam_if_source_deleted(Config) ->
     AppDir = ?config(apps, Config),
@@ -1272,44 +1260,424 @@ clean_all(Config) ->
                                          {app, PkgName, invalid}]}).
 
 override_deps(Config) ->
-    mock_git_resource:mock([{deps, [{some_dep, "0.0.1"},{other_dep, "0.0.1"}]}]),
     Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
     TopDeps = rebar_test_utils:top_level_deps(Deps),
 
-    RebarConfig = [
-        {deps, TopDeps},
-        {overrides, [
-            {override, some_dep, [
-                                 {deps, []}
-                                 ]}
-                    ]}
-        ],
-    rebar_test_utils:run_and_check(
-        Config, RebarConfig, ["compile"],
-        {ok, [{dep, "some_dep"},{dep_not_exist, "other_dep"}]}
-    ).
-
-profile_override_deps(Config) ->
-    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
-    TopDeps = rebar_test_utils:top_level_deps(Deps),
     {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
     mock_git_resource:mock([{deps, SrcDeps}]),
 
     RebarConfig = [
         {deps, TopDeps},
-        {profiles, [{a,
-                    [{overrides, [
-                                {override, some_dep, [
-                                                     {deps, []}
-                                                     ]}
-                                ]}
+        {overrides, [
+            {override, some_dep, [
+                {deps, []}
+            ]}
+        ]}
+    ],
+    rebar_test_utils:run_and_check(
+        Config, RebarConfig, ["compile"],
+        {ok, [{dep, "some_dep"},
+              {dep_not_exist, "other_dep"}]}
+    ).
+
+override_add_deps(Config) ->
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+
+    DepA = {dep_a, "0.0.1", {git, "http://site.com/dep_a.git", {tag, "0.0.1"}}},
+    DepB = {dep_b, "0.0.1", {git, "http://site.com/dep_b.git", {tag, "0.0.1"}}},
+    DepC = {dep_c, "0.0.1", {git, "http://site.com/dep_c.git", {tag, "0.0.1"}}},
+
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, [DepA, DepB, DepC | SrcDeps]}]),
+
+    RebarConfig = [
+        {deps, TopDeps},
+        {overrides, [
+            {add, some_dep, [
+                {deps, [DepA, DepB]}
+            ]},
+            {add, [
+                {deps, [DepC]}
+            ]}
+        ]}
+    ],
+    rebar_test_utils:run_and_check(
+        Config, RebarConfig, ["compile"],
+        {ok, [{dep, "some_dep"},
+              {dep, "other_dep"},
+              {dep, "dep_a"},
+              {dep, "dep_b"},
+              {dep, "dep_c"}]}
+    ).
+
+override_del_deps(Config) ->
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"dep_a", "0.0.1", []},
+                                                                     {"dep_b", "0.0.1", []},
+                                                                     {"dep_c", "0.0.1", []}]},
+                                              {"other_dep", "0.0.1", [{"dep_c", "0.0.1", []},
+                                                                      {"dep_d", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+
+    DepA = {dep_a, "0.0.1", {git, "https://example.org/user/dep_a.git", {tag, "0.0.1"}}},
+    DepB = {dep_b, "0.0.1", {git, "https://example.org/user/dep_b.git", {tag, "0.0.1"}}},
+    DepC = {dep_c, "0.0.1", {git, "https://example.org/user/dep_c.git", {tag, "0.0.1"}}},
+
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, SrcDeps}]),
+
+    RebarConfig = [
+        {deps, TopDeps},
+        {overrides, [
+            {del, some_dep, [
+                {deps, [DepA, DepB]}
+            ]},
+            {del, [
+                {deps, [DepC]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:run_and_check(
+        Config, RebarConfig, ["compile"],
+        {ok, [{dep, "some_dep"},
+              {dep, "other_dep"},
+              {dep_not_exist, "dep_a"},
+              {dep_not_exist, "dep_b"},
+              {dep_not_exist, "dep_c"},
+              {dep, "dep_d"}]}
+    ).
+
+override_opts(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [
+        {erl_opts, [
+            compressed,
+            warn_missing_spec
+        ]},
+        {overrides, [
+            {override, [
+                {erl_opts, [compressed]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+         Config, RebarConfig, ["compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "default", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    false = lists:member(warn_missing_spec, proplists:get_value(options, Mod:module_info(compile), [])).
+
+%% test for fix of https://github.com/erlang/rebar3/issues/1801
+%% only apply overrides once
+%% verify by having an override add the macro TEST to the dep some_dep
+%% building under `ct` will fail if the `add` is applied more than once
+apply_overrides_exactly_once(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, SrcDeps}]),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [{deps, TopDeps},
+                   {overrides, [
+                                {add, some_dep, [
+                                             {erl_opts, [{d, 'TEST'}]}
+                                            ]}
+                               ]}],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+      Config, RebarConfig, ["ct", "--compile_only"], {ok, [{app, Name}, {dep, "some_dep"}], "test"}).
+
+override_add_opts(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [
+        {erl_opts, [
+            warn_missing_spec
+        ]},
+        {overrides, [
+            {add, [
+                {erl_opts, [compressed]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+         Config, RebarConfig, ["compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "default", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    true = lists:member(warn_missing_spec, proplists:get_value(options, Mod:module_info(compile), [])).
+
+override_del_opts(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [
+        {erl_opts, [
+            compressed,
+            warn_missing_spec
+        ]},
+        {overrides, [
+            {del, [
+                {erl_opts, [warn_missing_spec]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+         Config, RebarConfig, ["compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "default", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    false = lists:member(warn_missing_spec, proplists:get_value(options, Mod:module_info(compile), [])).
+
+profile_override_deps(Config) ->
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, SrcDeps}]),
+
+    RebarConfig = [
+        {deps, TopDeps},
+        {profiles, [
+            {a, [
+                {overrides, [
+                    {override, some_dep, [
+                        {deps, []}
                     ]}
+                ]}
+            ]}
         ]}],
     rebar_test_utils:run_and_check(
         Config, RebarConfig, ["as", "a", "compile"],
-        {ok, [{dep, "some_dep"},{dep_not_exist, "other_dep"}]}
+        {ok, [{dep, "some_dep"},
+              {dep_not_exist, "other_dep"}]}
     ).
 
+profile_override_add_deps(Config) ->
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+
+    DepA = {dep_a, "0.0.1", {git, "http://site.com/dep_a.git", {tag, "0.0.1"}}},
+    DepB = {dep_b, "0.0.1", {git, "http://site.com/dep_b.git", {tag, "0.0.1"}}},
+    DepC = {dep_c, "0.0.1", {git, "http://site.com/dep_c.git", {tag, "0.0.1"}}},
+
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, [DepA, DepB, DepC | SrcDeps]}]),
+
+    RebarConfig = [
+        {deps, TopDeps},
+        {profiles, [
+            {a, [
+                {overrides, [
+                    {add, some_dep, [
+                        {deps, [DepA, DepB]}
+                    ]},
+                    {add, [
+                        {deps, [DepC]}
+                    ]}
+                ]}
+            ]}
+        ]}
+    ],
+    rebar_test_utils:run_and_check(
+        Config, RebarConfig, ["as", "a", "compile"],
+        {ok, [{dep, "some_dep"},
+              {dep, "other_dep"},
+              {dep, "dep_a"},
+              {dep, "dep_b"},
+              {dep, "dep_c"}]}
+    ).
+
+profile_override_del_deps(Config) ->
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"dep_a", "0.0.1", []},
+                                                                     {"dep_b", "0.0.1", []},
+                                                                     {"dep_c", "0.0.1", []}]},
+                                              {"other_dep", "0.0.1", [{"dep_c", "0.0.1", []},
+                                                                      {"dep_d", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+
+    DepA = {dep_a, "0.0.1", {git, "https://example.org/user/dep_a.git", {tag, "0.0.1"}}},
+    DepB = {dep_b, "0.0.1", {git, "https://example.org/user/dep_b.git", {tag, "0.0.1"}}},
+    DepC = {dep_c, "0.0.1", {git, "https://example.org/user/dep_c.git", {tag, "0.0.1"}}},
+
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, SrcDeps}]),
+
+    RebarConfig = [
+        {deps, TopDeps},
+        {profiles, [
+            {a, [
+                {overrides, [
+                    {del, some_dep, [
+                        {deps, [DepA, DepB]}
+                    ]},
+                    {del, [
+                        {deps, [DepC]}
+                    ]}
+                ]}
+            ]}
+        ]}
+    ],
+    rebar_test_utils:run_and_check(
+        Config, RebarConfig, ["as", "a", "compile"],
+        {ok, [{dep, "some_dep"},
+              {dep, "other_dep"},
+              {dep_not_exist, "dep_a"},
+              {dep_not_exist, "dep_b"},
+              {dep_not_exist, "dep_c"},
+              {dep, "dep_d"}]}
+    ).
+
+profile_override_opts(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [
+        {erl_opts, [
+            compressed,
+            warn_missing_spec
+        ]},
+        {profiles, [
+            {a, [
+                {overrides, [
+                    {override, [
+                        {erl_opts, [compressed]}
+                    ]}
+                ]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+         Config, RebarConfig, ["as", "a", "compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "a", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    false = lists:member(warn_missing_spec, proplists:get_value(options, Mod:module_info(compile), [])).
+
+profile_override_add_opts(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [
+        {erl_opts, [
+            warn_missing_spec
+        ]},
+        {profiles, [
+            {a, [
+                {overrides, [
+                    {add, [
+                        {erl_opts, [compressed]}
+                    ]}
+                ]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+         Config, RebarConfig, ["as", "a", "compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "a", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    true = lists:member(warn_missing_spec, proplists:get_value(options, Mod:module_info(compile), [])).
+
+profile_override_del_opts(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    RebarConfig = [
+        {erl_opts, [
+            compressed,
+            warn_missing_spec
+        ]},
+        {profiles, [
+            {a, [
+                {overrides, [
+                    {del, [
+                        {erl_opts, [warn_missing_spec]}
+                    ]}
+                ]}
+            ]}
+        ]}
+    ],
+
+    rebar_test_utils:create_config(AppDir, RebarConfig),
+
+    rebar_test_utils:run_and_check(
+         Config, RebarConfig, ["as", "a", "compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "a", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    false = lists:member(warn_missing_spec, proplists:get_value(options, Mod:module_info(compile), [])).
+
 profile_deps(Config) ->
     Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
     TopDeps = rebar_test_utils:top_level_deps(Deps),
@@ -1324,6 +1692,25 @@ profile_deps(Config) ->
         {ok, [{dep, "some_dep"},{dep, "other_dep"}]}
     ).
 
+only_deps(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("app1_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    Deps = rebar_test_utils:expand_deps(git, [{"some_dep", "0.0.1", [{"other_dep", "0.0.1", []}]}]),
+    TopDeps = rebar_test_utils:top_level_deps(Deps),
+    {SrcDeps, _} = rebar_test_utils:flat_deps(Deps),
+    mock_git_resource:mock([{deps, SrcDeps}]),
+
+    RConfFile = rebar_test_utils:create_config(AppDir, [{deps, TopDeps}]),
+    {ok, RConf} = file:consult(RConfFile),
+    rebar_test_utils:run_and_check(
+        Config, RConf, ["compile", "--deps_only"],
+        {ok, [{app_not_exist, Name}, {dep, "some_dep"},{dep, "other_dep"}]}
+    ).
+
 %% verify a deps prod profile is used
 %% tested by checking prod hooks run and outputs to default profile dir for dep
 %% and prod deps are installed for dep
@@ -1488,7 +1875,7 @@ include_file_in_src_test_multiapp(Config) ->
     AppDir1 = filename:join([?config(apps, Config), "lib", Name1]),
     AppDir2 = filename:join([?config(apps, Config), "lib", Name2]),
     Vsn = rebar_test_utils:create_random_vsn(),
-    rebar_test_utils:create_app(AppDir1, Name1, Vsn, [kernel, stdlib]),
+    rebar_test_utils:create_app(AppDir1, Name1, Vsn, [kernel, stdlib, list_to_atom(Name2)]),
     rebar_test_utils:create_app(AppDir2, Name2, Vsn, [kernel, stdlib]),
 
     Src = "-module(test).\n"
@@ -1510,7 +1897,8 @@ include_file_in_src_test_multiapp(Config) ->
     RebarConfig = [],
     rebar_test_utils:run_and_check(Config, RebarConfig,
                                    ["as", "test", "compile"],
-                                   {ok, [{app, Name1}]}).
+                                   {ok, [{app, Name1}]}),
+    ok.
 
 %% this test sets the env var, compiles, records the file last modified timestamp,
 %% recompiles and compares the file last modified timestamp to ensure it hasn't
@@ -1594,6 +1982,29 @@ recompile_when_erl_compiler_options_env_changes(Config) ->
         _     -> os:putenv("ERL_COMPILER_OPTIONS", ExistingEnv)
     end.
 
+rebar_config_os_var(Config) ->
+    AppDir = ?config(apps, Config),
+
+    Name = rebar_test_utils:create_random_name("rebar_config_os_var_"),
+    Vsn = rebar_test_utils:create_random_vsn(),
+    rebar_test_utils:create_app(AppDir, Name, Vsn, [kernel, stdlib]),
+
+    rebar_test_utils:create_config(AppDir, [{erl_opts, []}]),
+
+    AltConfig = filename:join(AppDir, "test.rebar.config"),
+    file:write_file(AltConfig, "{erl_opts, [compressed]}."),
+    true = os:putenv("REBAR_CONFIG", AltConfig),
+
+    rebar_test_utils:run_and_check(Config, ["compile"], {ok, [{app, Name}]}),
+
+    Path = filename:join([AppDir, "_build", "default", "lib", Name, "ebin"]),
+    code:add_patha(Path),
+
+    Mod = list_to_atom("not_a_real_src_" ++ Name),
+
+    true = lists:member(compressed, proplists:get_value(options, Mod:module_info(compile), [])),
+    ok.
+
 %% this test sets the env var, compiles, records the file last modified
 %% timestamp, recompiles and compares the file last modified timestamp to
 %% ensure it has changed. this test should run on 18.x
@@ -1746,7 +2157,13 @@ recursive(Config) ->
 
     EbinDir = filename:join([AppDir, "_build", "default", "lib", Name, "ebin"]),
     {ok, Files} = rebar_utils:list_dir(EbinDir),
-    ?assert(lists:member("rec.beam",Files)).
+    ?assert(lists:member("rec.beam",Files)),
+
+    %% check that rec is in modules list of .app file
+    AppFile = filename:join(EbinDir, Name++".app"),
+    {ok, [{application, _, List}]} = file:consult(AppFile),
+    {modules, Modules} = lists:keyfind(modules, 1, List),
+    ?assert(lists:member(rec, Modules)).
 
 no_recursive(Config) ->
     AppDir = ?config(apps, Config),
@@ -1803,3 +2220,143 @@ regex_filter_regression(Config) ->
                                    {ok, [{file, Expected}]}),
     ok.
 
+%%
+
+%% a copy of lib/parsetools/include/yeccpre.hrl so we can test yrl includefile
+yeccpre_hrl() ->
+    <<"-type yecc_ret() :: {'error', _} | {'ok', _}.
+
+-spec parse(Tokens :: list()) -> yecc_ret().
+parse(Tokens) ->
+    yeccpars0(Tokens, {no_func, no_line}, 0, [], []).
+
+-spec parse_and_scan({function() | {atom(), atom()}, [_]}
+                     | {atom(), atom(), [_]}) -> yecc_ret().
+parse_and_scan({F, A}) ->
+    yeccpars0([], {{F, A}, no_line}, 0, [], []);
+parse_and_scan({M, F, A}) ->
+    Arity = length(A),
+    yeccpars0([], {{fun M:F/Arity, A}, no_line}, 0, [], []).
+
+-spec format_error(any()) -> [char() | list()].
+format_error(Message) ->
+    case io_lib:deep_char_list(Message) of
+        true ->
+            Message;
+        _ ->
+            io_lib:write(Message)
+    end.
+
+%% To be used in grammar files to throw an error message to the parser
+%% toplevel. Doesn't have to be exported!
+-compile({nowarn_unused_function, return_error/2}).
+-spec return_error(integer(), any()) -> no_return().
+return_error(Line, Message) ->
+    throw({error, {Line, ?MODULE, Message}}).
+
+-define(CODE_VERSION, \"1.4\").
+
+yeccpars0(Tokens, Tzr, State, States, Vstack) ->
+    try yeccpars1(Tokens, Tzr, State, States, Vstack)
+    catch
+        error:Error ->
+            try yecc_error_type(Error, []) of
+                Desc ->
+                    erlang:raise(error, {yecc_bug, ?CODE_VERSION, Desc},
+                                 [])
+            catch _:_ -> erlang:raise(error, Error, [])
+            end;
+        %% Probably thrown from return_error/2:
+        throw: {error, {_Line, ?MODULE, _M}} = Error ->
+            Error
+    end.
+
+yecc_error_type(function_clause, _) ->
+    not_implemented.
+
+yeccpars1([Token | Tokens], Tzr, State, States, Vstack) ->
+    yeccpars2(State, element(1, Token), States, Vstack, Token, Tokens, Tzr);
+yeccpars1([], {{F, A},_Line}, State, States, Vstack) ->
+    case apply(F, A) of
+        {ok, Tokens, Endline} ->
+            yeccpars1(Tokens, {{F, A}, Endline}, State, States, Vstack);
+        {eof, Endline} ->
+            yeccpars1([], {no_func, Endline}, State, States, Vstack);
+        {error, Descriptor, _Endline} ->
+            {error, Descriptor}
+    end;
+yeccpars1([], {no_func, no_line}, State, States, Vstack) ->
+    Line = 999999,
+    yeccpars2(State, '$end', States, Vstack, yecc_end(Line), [],
+              {no_func, Line});
+yeccpars1([], {no_func, Endline}, State, States, Vstack) ->
+    yeccpars2(State, '$end', States, Vstack, yecc_end(Endline), [],
+              {no_func, Endline}).
+
+%% yeccpars1/7 is called from generated code.
+%%
+%% When using the {includefile, Includefile} option, make sure that
+%% yeccpars1/7 can be found by parsing the file without following
+%% include directives. yecc will otherwise assume that an old
+%% yeccpre.hrl is included (one which defines yeccpars1/5).
+yeccpars1(State1, State, States, Vstack, Token0, [Token | Tokens], Tzr) ->
+    yeccpars2(State, element(1, Token), [State1 | States],
+              [Token0 | Vstack], Token, Tokens, Tzr);
+yeccpars1(State1, State, States, Vstack, Token0, [], {{_F,_A}, _Line}=Tzr) ->
+    yeccpars1([], Tzr, State, [State1 | States], [Token0 | Vstack]);
+yeccpars1(State1, State, States, Vstack, Token0, [], {no_func, no_line}) ->
+    Line = yecctoken_end_location(Token0),
+    yeccpars2(State, '$end', [State1 | States], [Token0 | Vstack],
+              yecc_end(Line), [], {no_func, Line});
+yeccpars1(State1, State, States, Vstack, Token0, [], {no_func, Line}) ->
+    yeccpars2(State, '$end', [State1 | States], [Token0 | Vstack],
+              yecc_end(Line), [], {no_func, Line}).
+
+%% For internal use only.
+yecc_end({Line,_Column}) ->
+    {'$end', Line};
+yecc_end(Line) ->
+    {'$end', Line}.
+
+yecctoken_end_location(Token) ->
+    try erl_anno:end_location(element(2, Token)) of
+        undefined -> yecctoken_location(Token);
+        Loc -> Loc
+    catch _:_ -> yecctoken_location(Token)
+    end.
+
+-compile({nowarn_unused_function, yeccerror/1}).
+yeccerror(Token) ->
+    Text = yecctoken_to_string(Token),
+    Location = yecctoken_location(Token),
+    {error, {Location, ?MODULE, [\"syntax error before: \", Text]}}.
+
+-compile({nowarn_unused_function, yecctoken_to_string/1}).
+yecctoken_to_string(Token) ->
+    try erl_scan:text(Token) of
+        undefined -> yecctoken2string(Token);
+        Txt -> Txt
+    catch _:_ -> yecctoken2string(Token)
+    end.
+
+yecctoken_location(Token) ->
+    try erl_scan:location(Token)
+    catch _:_ -> element(2, Token)
+    end.
+
+-compile({nowarn_unused_function, yecctoken2string/1}).
+yecctoken2string({atom, _, A}) -> io_lib:write_atom(A);
+yecctoken2string({integer,_,N}) -> io_lib:write(N);
+yecctoken2string({float,_,F}) -> io_lib:write(F);
+yecctoken2string({char,_,C}) -> io_lib:write_char(C);
+yecctoken2string({var,_,V}) -> io_lib:format(\"~s\", [V]);
+yecctoken2string({string,_,S}) -> io_lib:write_string(S);
+yecctoken2string({reserved_symbol, _, A}) -> io_lib:write(A);
+yecctoken2string({_Cat, _, Val}) -> io_lib:format(\"~p\", [Val]);
+yecctoken2string({dot, _}) -> \"'.'\";
+yecctoken2string({'$end', _}) -> [];
+yecctoken2string({Other, _}) when is_atom(Other) ->
+    io_lib:write_atom(Other);
+yecctoken2string(Other) ->
+    io_lib:format(\"~p\", [Other]).
+">>.
